// 
// This file is part of MolMeccano, a cross-platform C++ chemical library.
//
// Copyright (C) 2001-2007 Alexey Nikitin
//
#if defined WIN32
#pragma warning (disable: 4503)
#endif

#include "Atom_kit.h"

#include "Pdb_attributes.h"
#include "MOT.h"
#include "Molecule.h"
#include "Residue_impl.h"
#include "Atom.h"
#include "User.h"
#include "Fine_type_impl.h"
#include "Vector_3D_impl.h"

#include <string.h>

namespace MM
{

Atom_kit::
Atom_kit (Atom & atom)
:
    in_molecule_    (&Molecule::none()),
    in_residue_     (&Residue ::none()),
    mm_type_        (0),   //fix   -1
    number_         (-1),
    atom_           (atom),
    fine_type_      (Fine_type::none),
    potential_force_(0,0,0),
    velocity_       (0,0,0),
    overlap_flag_   (false),
    transparency_   (1.),
    cache___        (0)
{
    visual_.adopt (new Visual_atom_composite);
}

bool Atom_kit::
has_pdb_attributes () const
{
    return pdb_attributes_.was_initialized();
}

void Atom_kit::
add_pdb_attributes (Pdb_attributes * adoptee)
{
    REQUIRE ("",  !has_pdb_attributes());
    pdb_attributes_.adopt (adoptee);
    ENSURE  ("",  has_pdb_attributes());
}

void Atom_kit::
remove_pdb_attributes()
{
    REQUIRE ("",  has_pdb_attributes());
    delete pdb_attributes_.orphan();
    ENSURE  ("",  !has_pdb_attributes());
}


bool Atom_kit::
has_hyperchem_attributes() const
{
    return hyperchem_attributes_.was_initialized();
}

void Atom_kit::
add_hyperchem_attributes (Hyperchem_attributes * adoptee)
{
    //REQUIRE ("",  !has_hyperchem_attributes());
    hyperchem_attributes_.adopt (adoptee);
    ENSURE  ("",  has_hyperchem_attributes());
}

void Atom_kit::
remove_hyperchem_attributes()
{
    REQUIRE ("",  has_hyperchem_attributes());
    delete hyperchem_attributes_.orphan();
    ENSURE  ("",  !has_hyperchem_attributes());
}

bool Atom_kit::
has_visual () const
{
    return visual_.was_initialized();
}

void Atom_kit::
add_visual (Visual_atom *adoptee)
{
    visual_().adopt (adoptee);
}

bool Atom_kit::
has_mm_type () const
{
    return mm_type_ != Micro_object_type::undefined_type ||
           !mm_type_name_.is_empty ();
}

const char * Atom_kit::
mm_type_c_str    () const
{
    if (mm_type_ == Micro_object_type::undefined_type)
        return mm_type_name_.c_str();

    return ::mot().itoa (mm_type_);
}

void Atom_kit::
set_mm_type (const char * str)
{
    mm_type_ = ::mot().atoi (str);
    mm_type_name_ = str;
}
/*
const char * Atom_kit::
flag_c_str (char * str ) const
{
    strcpy( str, "" );
    if( flag_.is( Atom_kit::hetero )               ) strcat( str, "h" );
    if( flag_.is( Atom_kit::has_improper_torsion ) ) strcat( str, "i" );
    if( flag_.is( Atom_kit::united )               ) strcat( str, "x" );
    if( flag_.is( Atom_kit::selected )             ) strcat( str, "s" );

    if( strlen( str ) == 0 )  strcat( str, "-" );
    return str;
}

void Atom_kit::
set_flag (const char * str)
{
    int i, size = strlen (str);

    for (i = 0;  i < size;  i++)
    {
        switch (str[i])
        {
            case '-':
                flag_.clear (Atom_kit::hetero);
                flag_.clear (Atom_kit::has_improper_torsion);
                flag_.clear (Atom_kit::united);
                flag_.clear (Atom_kit::selected);
                break;

            case 'h':
                flag_.set (Atom_kit::hetero);
                break;
            case 'i':
                flag_.set (Atom_kit::has_improper_torsion);
                break;
            case 'x':
                flag_.set (Atom_kit::united);
                break;
            case 's':
                flag_.set (Atom_kit::selected);
                break;
            default :
                MM::to_user().error ("Atom_kit::set_flag");
        }
    }
}//*/

char const * Atom_kit::
fine_type_name () const
{
    //return Fine_type::singleton().name (fine_type_);
    return Fine_type::name (fine_type_);
}

void Atom_kit::
set_fine_type (char const* name)
{
    //fine_type_ = Fine_type::singleton().id (name);
    fine_type_ = Fine_type::id (name);
}

void Atom_kit::
set_potential_force (Vector_3D const & new_value)
{
    potential_force_.set_x (new_value.x());
    potential_force_.set_y (new_value.y());
    potential_force_.set_z (new_value.z());
}

void Atom_kit::
set_velocity (Vector_3D const & new_value)
{
    velocity_.set_x (new_value.x());
    velocity_.set_y (new_value.y());
    velocity_.set_z (new_value.z());
}

void Atom_kit::
add_potential_force (Vector_3D const & value)
{
    potential_force_ += value;
}

void Atom_kit::
add_velocity (Vector_3D const & value)
{
    velocity_ += value;
}

}//MM
