#include "Crystal_builder_impl.h"

#include "Log.h"
#include "Atom_kit.h"

#include "Defs.h"
#include "Vector_3D_impl.h"
#include "Point_3D_impl.h"
#include "Model.h"
#include "Model_kit.h"
#include "Atom.h"
#include "Box_impl.h"
#include "Mach_eps.h"
#include "Lock.h"

#include "Create.h"

#include <math.h>

namespace MM
{
Crystal_builder_impl::
Crystal_builder_impl (Prototype const &)
:
    use_PBC_(true), //show_images_(false), //fix
    a_copies_(2), b_copies_(2), c_copies_(2),
    model_(nil<Model>())
{
    INVARIANT;
}


Crystal_builder_impl::
Crystal_builder_impl (Model & model)
:
    use_PBC_(true), //show_images_(false), //fix
    a_copies_(2), b_copies_(2), c_copies_(2),
    model_(model)
{
    /*box().set (Vector_3D_impl(10,0,0),
               Vector_3D_impl(0,10,0),
               Vector_3D_impl(0,0,10));//fix to fit//*/
    fit_box ();
    INVARIANT;
}

Crystal_builder * Crystal_builder_impl::
clone (Model & model) const
{
    return new Crystal_builder_impl (model);
}

bool Crystal_builder_impl::
invariant () const
{
//    const double nil = 0.000001;
//    const double nil = ::d_mach_eps_2;

    bool result =
        !show_images()      || 
        ( 
            image_.x_size() == a_copies_ &&
            image_.y_size() == b_copies_ &&
            image_.z_size() == c_copies_ &&
            janitor_.size() == a_copies_*b_copies_*c_copies_-1
        )
        ;

    if (!result) log ()
        << "show_images ()       " << show_images() << "\n"
        << "image_.size ()       " << image_.size() << "\n"
        << "image_.x_size()==a_copies_   " 
            << image_.x_size() << "  " << a_copies_ << "\n"
        << "image_.y_size()==b_copies_   " 
            << image_.y_size() << "  " << b_copies_ << "\n"
        << "image_.z_size()==c_copies_   " 
            << image_.z_size() << "  " << c_copies_ << "\n"
        << "janitor_.size() == a_copies_*b_copies_*c_copies_-1   " 
            << janitor_.size() << "  " << a_copies_*b_copies_*c_copies_-1 << "\n"
        << "\n";

    return result;
}

Box & Crystal_builder_impl::
box () const
{
    return model_.kit().crystal_cell ();
}

Vector_3D const & Crystal_builder_impl::
a () const
{
    return box().a();
}

Vector_3D const & Crystal_builder_impl::
b () const
{
    return box().b();
}

Vector_3D const & Crystal_builder_impl::
c () const
{
    return box().c();
}

double Crystal_builder_impl::
a_mod () const
{
    return box().a().length();
}

double Crystal_builder_impl::
b_mod () const
{
    return box().b().length();
}

double Crystal_builder_impl::
c_mod () const
{
    return box().c().length();
}

void Crystal_builder_impl::
set (Vector_3D const & a, Vector_3D const & b, Vector_3D const & c)
{
    box().set (a, b, c);
    update_images();
}

double Crystal_builder_impl::
alpha () const
{
    INVARIANT;
    return box().alpha ();
}

double Crystal_builder_impl::
beta  () const
{
    INVARIANT;
    return box().beta ();
}

double Crystal_builder_impl::
gamma () const
{
    INVARIANT;
    return box().gamma ();
}

void Crystal_builder_impl::
set (double new_a,     double new_b,    double new_c,
     double new_alpha, double new_beta, double new_gamma)
{
    INVARIANT;
    box().set (new_a, new_b, new_c, new_alpha, new_beta, new_gamma);
    update_images ();
}

void Crystal_builder_impl::
show_cell_copies ()
{
    //CHECK ("show_images_ flag is on.", show_images());

    janitor_.clear();
    image_.resize (a_copies_, b_copies_, c_copies_);
    
    for (int i=1;  i<image_.size();  ++i)
    {
        Model *new_model = model_.clone();
        janitor_.adopt (new_model);
        image_[i] = new_model;
    }

    update_images ();
    image_[0] = &model_;

    INVARIANT;
}

void Crystal_builder_impl::
hide_cell_copies ()
{
    //INVARIANT;
    //REQUIRE ("show_images_ flag is off.", !show_images_);
    
    image_.clear();
    janitor_.clear();

    //INVARIANT;
}

void Crystal_builder_impl::
convert_from_fractional_to_cartesian ()
{
    // applay transposed matrix to each atom

    for (int i=0; i<model().atom_count();  ++i)
    {
        Atom &          current_atom     = model().atom (i);
        Point_3D const &current_position = current_atom.position();
        Point_3D_impl   new_position (0,0,0);

        new_position.set_x (box().a().x()*current_position.x()
                          + box().b().x()*current_position.y()
                          + box().c().x()*current_position.z());

        new_position.set_y (box().a().y()*current_position.x()
                          + box().b().y()*current_position.y()
                          + box().c().y()*current_position.z());

        new_position.set_z (box().a().z()*current_position.x()
                          + box().b().z()*current_position.y()
                          + box().c().z()*current_position.z());

        current_atom.set_position (new_position);
    }
}

void Crystal_builder_impl::
fit_box ()
{
    box().fit (model_, Box::Cuboid, 1.5);
}

void Crystal_builder_impl::
update_images ()
{
    if (!show_images())
        return;

    INVARIANT;

    int i, j, k, n;

    for (n=0;  n<model_.atom_count();  ++n)
    {
        Atom &current_atom = model_.atom (n);
        Point_3D const & base_position = current_atom.position();

        for (i=0;  i<image_.x_size();  ++i)
        for (j=0;  j<image_.y_size();  ++j)
        for (k=0;  k<image_.z_size();  ++k)
        {
            if (i==0 && j==0 && k==0) 
                continue;
        
            Model &current_model = *image_(i,j,k);
            Vector_3D_impl movement (0,0,0);
            movement.add_scaled (i, box().a());
            movement.add_scaled (j, box().b());
            movement.add_scaled (k, box().c());
            Point_3D_impl new_position (base_position);
            new_position += movement;

            Atom &current_atom = current_model.atom (n);
            current_atom.set_position (new_position);
        }
    }
    INVARIANT;
}

void Crystal_builder_impl::
set_number_of_copies (int a, int b, int c)
{
    //INVARIANT;

    a_copies_ = a;
    b_copies_ = b;
    c_copies_ = c;

    //update_copies();
    if (show_images())
    {
        hide_cell_copies ();
        show_cell_copies ();
    }
    else
        hide_cell_copies ();

    //log() << "show_images_ " << show_images_ << "\n";
    //log() << "janitor_.size() " << janitor_.size() << "    a*b*c " << (a*b*c) << "\n";
    //log() << "image_.x_size() " << image_.x_size() << "    a     " << a       << "\n";
    //log() << "image_.y_size() " << image_.y_size() << "    b     " << b       << "\n";
    //log() << "image_.z_size() " << image_.z_size() << "    c     " << c       << "\n";

    ENSURE ("", !show_images()               || ( 
                janitor_.size() == (a*b*c-1) &&
                image_.x_size() == a         && 
                image_.y_size() == b         && 
                image_.z_size() == c));
}

/*void Crystal_builder::
update_copies ()
{
    //INVARIANT;
    if (show_images())
    {
        hide_cell_copies ();
        show_cell_copies ();
    }
    else
        hide_cell_copies ();

    INVARIANT;
}//*/

void Crystal_builder_impl::
set_show_images (bool show)
{
    INVARIANT;

    //show_images_ = show;

    if (show)
        show_cell_copies ();
    else
        hide_cell_copies ();

    INVARIANT;
}

void Crystal_builder_impl::
expand_model () 
{
    INVARIANT;

    Lock<Model_event> lock(model_.kit().event());
    bool show = show_images ();
    set_show_images (true);

    janitor_.orphan_all ();
    
    for (int i=1;  i<image_.size();  ++i)
        model_.merge_model (image_[i]); 

    box().scale_a (a_copies_);
    box().scale_b (b_copies_);
    box().scale_c (c_copies_);
     
    //set_number_of_copies (1,1,1);
    show_cell_copies ();
    set_show_images (show);

    INVARIANT;
}

}//MM












